>From f4b6d8876a269c0847d0d5b749ee673d0275b5aa Mon Sep 17 00:00:00 2001
From: Andres Freund <andres@anarazel.de>
Date: Thu, 6 Aug 2015 12:53:56 +0200
Subject: [PATCH] Don't include low level locking code from frontend code.

Sinc eea9df812d lwlock.h also includes s_lock.h, and ab5194e6 extended
that to atomics.h. It's not good that frontend code if frontend code
includes that, so rearrange things that it doesnt' anymore.

To avoid that happening again put some checks agains this happening into
atomics.h, lwlock.h and s_lock.h.
---
 contrib/pg_stat_statements/pg_stat_statements.c | 1 +
 src/backend/access/heap/syncscan.c              | 1 +
 src/backend/access/nbtree/nbtutils.c            | 1 +
 src/backend/access/transam/twophase.c           | 1 +
 src/backend/access/transam/varsup.c             | 1 +
 src/backend/access/transam/xact.c               | 1 +
 src/backend/access/transam/xlog.c               | 1 +
 src/backend/commands/tablespace.c               | 1 +
 src/backend/replication/logical/origin.c        | 1 +
 src/backend/utils/cache/relcache.c              | 1 +
 src/include/port/atomics.h                      | 4 ++++
 src/include/storage/lock.h                      | 1 -
 src/include/storage/lwlock.h                    | 4 ++++
 src/include/storage/proc.h                      | 1 +
 src/include/storage/s_lock.h                    | 4 ++++
 15 files changed, 23 insertions(+), 1 deletion(-)

diff --git a/contrib/pg_stat_statements/pg_stat_statements.c b/contrib/pg_stat_statements/pg_stat_statements.c
index 59b8a2e..3839b46 100644
--- a/contrib/pg_stat_statements/pg_stat_statements.c
+++ b/contrib/pg_stat_statements/pg_stat_statements.c
@@ -72,6 +72,7 @@
 #include "pgstat.h"
 #include "storage/fd.h"
 #include "storage/ipc.h"
+#include "storage/lwlock.h"
 #include "storage/spin.h"
 #include "tcop/utility.h"
 #include "utils/builtins.h"
diff --git a/src/backend/access/heap/syncscan.c b/src/backend/access/heap/syncscan.c
index 266c330..9a3b94b 100644
--- a/src/backend/access/heap/syncscan.c
+++ b/src/backend/access/heap/syncscan.c
@@ -48,6 +48,7 @@
 
 #include "access/heapam.h"
 #include "miscadmin.h"
+#include "storage/lwlock.h"
 #include "utils/rel.h"
 
 
diff --git a/src/backend/access/nbtree/nbtutils.c b/src/backend/access/nbtree/nbtutils.c
index 91331ba..fa9b87b 100644
--- a/src/backend/access/nbtree/nbtutils.c
+++ b/src/backend/access/nbtree/nbtutils.c
@@ -21,6 +21,7 @@
 #include "access/reloptions.h"
 #include "access/relscan.h"
 #include "miscadmin.h"
+#include "storage/lwlock.h"
 #include "utils/array.h"
 #include "utils/lsyscache.h"
 #include "utils/memutils.h"
diff --git a/src/backend/access/transam/twophase.c b/src/backend/access/transam/twophase.c
index 177d1e1..952ef98 100644
--- a/src/backend/access/transam/twophase.c
+++ b/src/backend/access/transam/twophase.c
@@ -60,6 +60,7 @@
 #include "replication/syncrep.h"
 #include "storage/fd.h"
 #include "storage/ipc.h"
+#include "storage/lwlock.h"
 #include "storage/predicate.h"
 #include "storage/proc.h"
 #include "storage/procarray.h"
diff --git a/src/backend/access/transam/varsup.c b/src/backend/access/transam/varsup.c
index cf3e964..428c3ff 100644
--- a/src/backend/access/transam/varsup.c
+++ b/src/backend/access/transam/varsup.c
@@ -22,6 +22,7 @@
 #include "commands/dbcommands.h"
 #include "miscadmin.h"
 #include "postmaster/autovacuum.h"
+#include "storage/lwlock.h"
 #include "storage/pmsignal.h"
 #include "storage/proc.h"
 #include "utils/syscache.h"
diff --git a/src/backend/access/transam/xact.c b/src/backend/access/transam/xact.c
index b53d95f..732b14e 100644
--- a/src/backend/access/transam/xact.c
+++ b/src/backend/access/transam/xact.c
@@ -47,6 +47,7 @@
 #include "replication/origin.h"
 #include "storage/fd.h"
 #include "storage/lmgr.h"
+#include "storage/lwlock.h"
 #include "storage/predicate.h"
 #include "storage/proc.h"
 #include "storage/procarray.h"
diff --git a/src/backend/access/transam/xlog.c b/src/backend/access/transam/xlog.c
index 68e33eb..1568888 100644
--- a/src/backend/access/transam/xlog.c
+++ b/src/backend/access/transam/xlog.c
@@ -56,6 +56,7 @@
 #include "storage/ipc.h"
 #include "storage/large_object.h"
 #include "storage/latch.h"
+#include "storage/lwlock.h"
 #include "storage/pmsignal.h"
 #include "storage/predicate.h"
 #include "storage/proc.h"
diff --git a/src/backend/commands/tablespace.c b/src/backend/commands/tablespace.c
index ff0d904..1f56b51 100644
--- a/src/backend/commands/tablespace.c
+++ b/src/backend/commands/tablespace.c
@@ -73,6 +73,7 @@
 #include "postmaster/bgwriter.h"
 #include "storage/fd.h"
 #include "storage/lmgr.h"
+#include "storage/lwlock.h"
 #include "storage/standby.h"
 #include "utils/acl.h"
 #include "utils/builtins.h"
diff --git a/src/backend/replication/logical/origin.c b/src/backend/replication/logical/origin.c
index f4ba86e..49f5476 100644
--- a/src/backend/replication/logical/origin.c
+++ b/src/backend/replication/logical/origin.c
@@ -88,6 +88,7 @@
 #include "storage/fd.h"
 #include "storage/ipc.h"
 #include "storage/lmgr.h"
+#include "storage/lwlock.h"
 #include "storage/copydir.h"
 
 #include "utils/builtins.h"
diff --git a/src/backend/utils/cache/relcache.c b/src/backend/utils/cache/relcache.c
index 44e9509..d4b19cd 100644
--- a/src/backend/utils/cache/relcache.c
+++ b/src/backend/utils/cache/relcache.c
@@ -66,6 +66,7 @@
 #include "rewrite/rewriteDefine.h"
 #include "rewrite/rowsecurity.h"
 #include "storage/lmgr.h"
+#include "storage/lwlock.h"
 #include "storage/smgr.h"
 #include "utils/array.h"
 #include "utils/builtins.h"
diff --git a/src/include/port/atomics.h b/src/include/port/atomics.h
index bb87945..65cfa3f 100644
--- a/src/include/port/atomics.h
+++ b/src/include/port/atomics.h
@@ -37,6 +37,10 @@
 #ifndef ATOMICS_H
 #define ATOMICS_H
 
+#ifdef FRONTEND
+#error "atomics.h may not be included from frontend code"
+#endif
+
 #define INSIDE_ATOMICS_H
 
 #include <limits.h>
diff --git a/src/include/storage/lock.h b/src/include/storage/lock.h
index 96fe3a6..fb731b0 100644
--- a/src/include/storage/lock.h
+++ b/src/include/storage/lock.h
@@ -15,7 +15,6 @@
 #define LOCK_H_
 
 #include "storage/backendid.h"
-#include "storage/lwlock.h"
 #include "storage/shmem.h"
 
 
diff --git a/src/include/storage/lwlock.h b/src/include/storage/lwlock.h
index cbd6318..f2ff6a0 100644
--- a/src/include/storage/lwlock.h
+++ b/src/include/storage/lwlock.h
@@ -14,6 +14,10 @@
 #ifndef LWLOCK_H
 #define LWLOCK_H
 
+#ifdef FRONTEND
+#error "lwlock.h may not be included from frontend code"
+#endif
+
 #include "lib/ilist.h"
 #include "storage/s_lock.h"
 #include "port/atomics.h"
diff --git a/src/include/storage/proc.h b/src/include/storage/proc.h
index 202a672..fcbd9a5 100644
--- a/src/include/storage/proc.h
+++ b/src/include/storage/proc.h
@@ -18,6 +18,7 @@
 #include "lib/ilist.h"
 #include "storage/latch.h"
 #include "storage/lock.h"
+#include "storage/lwlock.h"
 #include "storage/pg_sema.h"
 
 /*
diff --git a/src/include/storage/s_lock.h b/src/include/storage/s_lock.h
index 30f28b0..c461fda 100644
--- a/src/include/storage/s_lock.h
+++ b/src/include/storage/s_lock.h
@@ -96,6 +96,10 @@
 #ifndef S_LOCK_H
 #define S_LOCK_H
 
+#ifdef FRONTEND
+#error "s_lock.h may not be included from frontend code"
+#endif
+
 #ifdef HAVE_SPINLOCKS	/* skip spinlocks if requested */
 
 #if defined(__GNUC__) || defined(__INTEL_COMPILER)
-- 
2.3.0.149.gf3f4077.dirty

